#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
	@author : Mirco Tracolli
	@version : 0.6
	
"""

import pymunk
import cocos
from sys import maxint
from math import sqrt,hypot,atan2,degrees
import random

random.seed()

pathatlas = 'data/zombie/'

class RespawnNPC():
	def __init__(self,space,layer):
		self.space = space
		self.layer = layer
		self.space.add_collision_handler(1, 2, self.bulletcollide, None, None, None)
		self.space.add_collision_handler(4, 2, None , None , self.playercollide , None)
		self.space.add_collision_handler(4, 3, None , self.playercolliderad , None , None)
		self.space.add_collision_handler(5, 3, None , self.engagetarget , None , None)
	
	# Genera n nemici dalle coordinate x,y che seguono il percorso route
	# Dir indica in che direzione generarli :
	# 0 = verso sinistra
	# 1 = verso destra
	# 2 = verso l'alto
	# 3 = verso il basso
	def gen_enemies(self,args):
		random.seed()
		enemies = {}
		if args['n'] > 0 :
			for en in range(0,args['n']):
				self.range_gen=args['range_gen']
				x=random.randint(self.range_gen[0],self.range_gen[1])# x to x
				y=random.randint(self.range_gen[2],self.range_gen[3])# y to y
				x+=args['xy'][0]
				y+=args['xy'][1]
				numberZ = str(random.randint(1,5))
				randomimage = [pathatlas+'zombie'+numberZ+".png",
								pathatlas+'zombie'+numberZ+"_dead.png"]
				argsNPC = {
							'player' : args['player'],
							'img' : randomimage,
							'layer' : self.layer,
							'depth' : args['depth'],
							'space' : self.space,
							'origin' : (x,y),
							'targets' : args['route'],
							'vel' : args['vel']+random.randint(-50,0),
							'seek' : args['seek']+random.uniform(-0.01,0.3),
							'time_walk' : args['time_walk'],
							'updown' : args['updown'],
							'smooth' : args['smooth'],
							'life' : 10+random.randint(0,30)
							}
				temp = NPC(argsNPC)
				enemies[id(temp)] = temp
				self.layer.add(temp.sprite,z = args['depth'], name = str(id(temp)))
		return(enemies)

	# Collisione con il proiettile
	def bulletcollide(self,space,arb):
		arb._get_shapes()[1].body.life -= 10
		arb._get_shapes()[0].body.callremove(arb._get_shapes()[0].body)
		if arb._get_shapes()[1].body.life <= 0 :
			arb._get_shapes()[0].body.player.point += arb._get_shapes()[1].body.enemypoints
			arb._get_shapes()[1].body.callremove(arb._get_shapes()[1].body)
		return(False)
	
	# Collisione con il nemico
	def playercollide(self,space,arb):
		arb._get_shapes()[0].body.player.life -= 1
	
	# Collisione con l'area radioattiva del nemico
	def playercolliderad(self,space,arb):
		arb._get_shapes()[0].body.player.life -= 0.1
		return(True)
		
	def engagetarget(self,space,arb):
		print ("Nemico Avvistato")
		return(True)

class NPC(pymunk.Body):
	
	def __init__(self,args):
		""" 
			space = ambiente fisico pymunk
			origin = posizione di partenza
			route = strada da percorrere
			vel = velocità di movimento
			seek = tempo di aggiornamento
			time_walk = tempo del passo
			updown = indica la direzione del movimento
					 True = ordinate
					 False = ascisse
			smooth = indica di voler avere un movimento fluido
		"""
		self.player = args['player']
		self.space = args['space']
		self.layer = args['layer']
		self.depth = args['depth']
		self.life = args['life']
		self.enemypoints = args['life']
		self.vel = args['vel']
		self.seek = args['seek']
		self.time = 0
		self.time_walk = args['time_walk']
		self.current_target = None
		self.updown = args['updown']
		self.smooth = args['smooth']
		self.pointN = 0
		
		self.sprite = cocos.sprite.Sprite(args['img'][0],scale=0.15)
		self.sprited = cocos.sprite.Sprite(args['img'][1],scale=0.15)
		self.sprite.position = args['origin']
		
		moment = pymunk.moment_for_circle(10,0,25)
		super(NPC,self).__init__(10,moment)
		self.position = args['origin']
		self.targets = args['targets']
		self.bodyshape = pymunk.Circle(self,25,(0,0))
		self.bodyshape._set_collision_type(2)
		self.bodyshape.friction = 1
		self.bodyshape._set_layers(2)
		
		self.bodyshape2 = pymunk.Circle(self,55,(0,0))
		self.bodyshape2._set_collision_type(3)
		self.bodyshape2.sensor = True
		
		self.space.add(self,self.bodyshape,self.bodyshape2)
	
	# Rimuove l'oggetto da una scena
	def remove(self,npc=None):
		self.sprited.position = self.position
		self.sprited.rotation = self.sprite.rotation
		self.sprite = self.sprited
		self.layer.remove(str(id(self)))
		depth = self.depth - 1.
		self.layer.add(self.sprited, z = depth, name = str(id(self)))
		if npc is not None :
			self.space.remove(npc.bodyshape,self.bodyshape2,npc)
		else :
			self.space.remove(self.bodyshape,self.bodyshape2,self)
	
	# Rimuove il proiettile dalla scena dopo una collisione
	def callremove(self,npc):
		self.space.add_post_step_callback(self.remove,npc)
	
	# Aggiorna lo status dell'npc
	def update(self,dt):
		self.targets[-1] = self.player.ccbody.position
		if self.life > 0 :
			self.sprite.position = self.position
			if int(self.sprite.rotation) != int(self.angle) :
						self.sprite.do( cocos.actions.RotateTo( self._get_angle(), 0.3 ) )
			if self.time < self.seek :
				self.time += dt
			else :
				if self.current_target != self.targets[0] :
					self.current_target = self.targets[0]
				if len(self.targets) > 1 and self.distance(self.position,self.targets[0]) < 150.0 :
					self.targets.remove(self.targets[0])
				if self.time_walk > 0 :
					self.time_walk -= dt
					if self.smooth :
						if self.velocity != (0,0) :
							dirmod = hypot(self.velocity[0],self.velocity[1])
							self.velocity = ((self.velocity[0]/dirmod)*(self.vel/3),(self.velocity[1]/dirmod)*(self.vel/3))
					else :
						self.velocity = (0,0)
				else:
					ver = self.getDir()
					if self.updown :
						self.velocity = (ver[0]*self.vel+random.randint(-25,25),ver[1]*self.vel)
					else :
						self.velocity = (ver[0]*self.vel,ver[1]*self.vel+random.randint(-25,25))
					self.time_walk = 0.01
					
				
				self.time = 0
		else :
			if self.time < self.seek*4:
				self.time += dt
			else :
				self.layer.remove(str(id(self)))
				return(False)
		return(True)

	# Ritorna il versore per lo spostamento
	def getDir(self):
		direction = (self.current_target[0] - self.position[0],self.current_target[1] - self.position[1])
		dirmod = hypot(direction[0],direction[1])
		if dirmod > 0 :
			rad = atan2(direction[0]/dirmod,direction[1]/dirmod)
			self.angle = degrees(rad)
			return((direction[0]/dirmod,direction[1]/dirmod))
		else :
			return((0,0))
		
	# Ritorna la distanza
	@staticmethod
	def distance(v1,v2):
		return(sqrt((v1[0]-v2[0])**2 + (v1[1]-v2[1])**2))
